# GNU Enterprise Forms - wx 2.6 UI Driver - ToolButton widget
#
# Copyright 2001-2007 Free Software Foundation
#
# This file is part of GNU Enterprise
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: toolbutton.py 9364 2007-02-09 11:17:51Z reinhard $

import gtk

from gnue.forms.uidrivers.gtk2.widgets._base import UIHelper


# =============================================================================
# UIToolButton
# =============================================================================

class UIToolButton(UIHelper):
    """
    Implements a toolbar button object.
    """

    # -------------------------------------------------------------------------
    # Create a menu item widget
    # -------------------------------------------------------------------------

    def _create_widget_(self, event, spacer):
        """
        Creates a new toolbar button widget.
        """

        # These are the relevant parameters
        icon_file = self._gfObject._get_icon_file(size="24x24", format="png")
        label = self._gfObject.label
        description = self._gfObject.description
        check = (self._gfObject.action_off is not None)

        if event.container is not None:
            if label is not None:
                if icon_file:
                    image = gtk.Image()
                    image.set_from_file(icon_file)
                else:
                    image = None

                if check:
                    widget = gtk.ToggleToolButton()
                    self.__h_toggled = widget.connect('toggled', self.__on_clicked)
                else:
                    widget = gtk.ToolButton()
                    widget.connect('clicked', self.__on_clicked)

                widget.set_icon_widget(image)
                widget.set_label(label)
                widget.set_tooltip(self._uiForm._tooltips, label, None)

                event.container.insert(widget, -1)

            else:
                widget = None
                event.container.insert(gtk.SeparatorToolItem(), -1)

        else:
            # TOOLBAR:SUPPRESS was set
            widget = None

        self.__widget = widget

        return widget


    # -------------------------------------------------------------------------
    # Events
    # -------------------------------------------------------------------------

    def __on_clicked(self, event):
        self._gfObject._event_fire()


    # -------------------------------------------------------------------------
    # Check/uncheck menu item
    # -------------------------------------------------------------------------

    def _ui_switch_on_(self):
        if self.__widget is not None:
            self.__widget.handler_block(self.__h_toggled)
            self.__widget.set_active(True)
            self.__widget.handler_unblock(self.__h_toggled)

    # -------------------------------------------------------------------------

    def _ui_switch_off_(self):
        if self.__widget is not None:
            self.__widget.handler_block(self.__h_toggled)
            self.__widget.set_active(False)
            self.__widget.handler_unblock(self.__h_toggled)


    # -------------------------------------------------------------------------
    # Enable/disable menu item
    # -------------------------------------------------------------------------

    def _ui_enable_(self):
        if self.__widget is not None:
            self.__widget.set_sensitive(True)

    # -------------------------------------------------------------------------

    def _ui_disable_(self):
        if self.__widget is not None:
            self.__widget.set_sensitive(False)


# =============================================================================
# Configuration data
# =============================================================================

configuration = {
  'baseClass': UIToolButton,
  'provides' : 'GFToolButton',
  'container': False
}
