# GNU Enterprise Forms - QT UI Driver - User Interface
#
# Copyright 2001-2007 Free Software Foundation
#
# This file is part of GNU Enterprise
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: UIdriver.py 9512 2007-04-24 19:15:08Z reinhard $
"""
A pyQT based user interface driver for GNUe forms.
"""

import os.path

from gnue.forms.uidrivers._base import Exceptions
from gnue.forms.uidrivers._base.UIdriver import GFUserInterfaceBase

try:
    from PyQt4 import Qt

except ImportError:
    raise Exceptions.DriverNotSupported, \
            u_("The GNUe-Forms qt4 driver requires PyQT 4.1+")

from gnue.common.apps import GConfig
from gnue.forms.uidrivers.qt4.QTApp import *
from gnue.forms.uidrivers.qt4 import dialogs


# =============================================================================
# Splash screen
# =============================================================================
class UISplashScreen(qt.QSplashScreen):
    """
    Splash screen window
    """

    # -------------------------------------------------------------------------
    # Constructor
    # -------------------------------------------------------------------------

    def __init__(self):

        idir = GConfig.getInstalledBase('forms_images', 'common_images')
        picture = gConfigForms ('splashScreenPNG')
        if not os.path.isabs(picture):
            picture = os.path.join (idir, picture)

        qt.QSplashScreen.__init__(self, qt.QPixmap(picture))

        self.show()
        getQtApp().processEvents()



# =============================================================================
# This class implements the User Interface for qt4
# =============================================================================

class GFUserInterface(GFUserInterfaceBase):

    __rearrange_boxes__ = True

    # -------------------------------------------------------------------------
    # Constructor
    # -------------------------------------------------------------------------

    def __init__(self, eventHandler, name="Undefined", disableSplash=None,
            parentContainer=None, moduleName=None):

        GFUserInterfaceBase.__init__(self, eventHandler, name,
                              disableSplash, parentContainer, moduleName)
        self.__splash = None

        # We don't need these values for this ui driver
        self.textWidth = self.widgetWidth = -1
        self.textHeight = self.widgetHeight = -1

        self.name = "qt4"

        self._qtapp = getQtApp()

        if not self._disableSplash:
            self.__splash = UISplashScreen()
        else:
            self.__splash = None



    # -------------------------------------------------------------------------
    # Start the application's main loop
    # -------------------------------------------------------------------------

    def mainLoop(self):
        """
        Start the main loop of the current application instance
        """

        assert gEnter(6)

        self._qtapp.start()

        assert gLeave(6)


    # -------------------------------------------------------------------------
    # Hide splash screen
    # -------------------------------------------------------------------------

    def hide_splash(self):
        """
        Hide the splash screen (if available)
        """

        if self.__splash:
            try:
                self.__splash.close()
                self._qtapp.processEvents()
            finally:
                self.__splash = None


    # -------------------------------------------------------------------------
    # Show a simple error message
    # -------------------------------------------------------------------------

    def _ui_show_error_(self, message):

        self.hide_splash()
        qt.QMessageBox.critical(None, "GNU Enterprise", message)


    # -------------------------------------------------------------------------
    # Show Exception dialog
    # -------------------------------------------------------------------------

    def _ui_show_exception_(self, group, name, message, detail):

        self.hide_splash()
        dlg = dialogs.ExceptionDialog(group, name, message, detail)
        dlg.exec_()

    # -------------------------------------------------------------------------
    # Create and run an input dialog
    # -------------------------------------------------------------------------

    def _getInput(self, title, fields, cancel=True):

        dlg = dialogs.InputDialog(title, fields, cancel)
        dlg.exec_()
        return dlg.inputData


    # -------------------------------------------------------------------------
    # Exit the application
    # -------------------------------------------------------------------------

    def _ui_exit_(self):

        assert gEnter(6)

        for child in self._children:
            child.main_window._closing_ = True
            child.main_window.close()

        assert gLeave(6)
